//游戏控制器
import Snake from './snake'
import Food from './food'
import Point from './point'

class GameControl {
  snake: Snake
  food: Food
  point: Point
  timeID: any

  // 存储移动的方向
  direction: string = ''

  // 游戏是否继续
  isLive: boolean = true

  constructor(){
    this.snake = new Snake()
    this.food = new Food()
    this.point = new Point(10, 3)
    this.init()
  }

  // 游戏初始化的方法
  init (){
    // 绑定键盘事件
    document.addEventListener('keydown',this.keydownHandle.bind(this))
    // document调用这个函数, 这里的this是document， 所以要么改成箭头函数，要么用bind

    this.run()
  }

  // 按键按下事件
   keydownHandle(e:KeyboardEvent){
    // console.log(this)
    // console.log(e.key);
    this.direction = e.key
  }

  // 蛇移动的方法
  run(){
    let X = this.snake.X
    let Y = this.snake.Y
    // 检查有没有吃到食物
    this.checkEat(X, Y)
    // 控制移动
    switch(this.direction) {
      // 上
      case "ArrowUp":
      case "Up": Y -= 10
        break
      //下
      case "ArrowDown":
      case "Down": Y += 10
        break
      // 左
      case "ArrowLeft":
      case "Left": X -= 10
        break
      // 右
      case "ArrowRight":
      case "Right": X += 10
        break
    }

    // 先检查有没有撞墙
    try{
      this.snake.X = X
      this.snake.Y = Y
    }catch(e){
      // 捕获到异常， 结束运行
      alert(e.message)
      clearTimeout(this.timeID)
      this.isLive = false
    }

    this.timeID && clearTimeout(this.timeID) // 短路运算
    if(this.isLive){
      this.timeID = setTimeout (this.run.bind(this), 300 - (this.point.level - 1) * 30)
    }
  }

  // 检查是否吃到食物
  checkEat(X:number, Y:number){
    if(X === this.food.X && Y === this.food.Y){
      console.log('吃到食物');
      this.food.change()
      this.point.addScore()
      this.snake.addBody()
    }
  }
}

export default GameControl