class Snake{
  element: HTMLElement
  // 蛇头
  head:HTMLElement
  // 蛇的身体(包括蛇头)
  bodies: HTMLCollection

  constructor(){
    this.element = document.getElementById('snake')!
    this.head = document.querySelector('#snake>div') as HTMLElement // 提示类型不匹配,断言
    this.bodies = document.getElementById('snake')!.getElementsByTagName('div')
    //query选择符选出来的元素及元素数组是静态的，而getElement这种方法选出的元素是动态的。静态的就是说选出的所有元素的数组，不会随着文档操作而改变．
    // 在使用的时候getElement这种方法性能比较好，query选择符则比较方便
  }

    // 蛇头的坐标
    get X(){
      return this.head.offsetLeft
    }
  
    get Y(){
      return this.head.offsetTop
    }

    // 设置蛇头的坐标
    set X(value: number){
      if(this.X == value) return

      // 蛇撞墙
      if(value < 0 || value > 290){
        throw new Error('蛇撞墙了')
      }

      // 没撞墙
      // 长度大于1时，禁止掉头
      if(this.bodies[1] && (this.bodies[1] as HTMLElement).offsetLeft === value){
        console.log("发生了掉头");
        if(value > this.X){
          // 向右掉头， 则要往左修正value
          value = this.X - 10
        } else {
          value = this.X + 10
        }
      }
      // 移动身体
      this.moveBody()  // 这里同样要先移动身体， 再移动头
      // 移动头
      this.head.style.left = value + 'px'
      // 检查头是否撞到了身体
      if(this.bodies.length > 4){
        this.checkBreak()
      }

    }

    set Y(value: number){
      if(this.Y == value) return
      // 蛇撞墙
      if(value < 0 || value > 290){
        throw new Error('蛇撞墙了')
      }
      // 没撞墙
      // 长度大于1 ， 禁止掉头
      if(this.bodies[1] && (this.bodies[1] as HTMLElement).offsetTop === value){
        console.log("发生了掉头");
        if(value > this.Y){
          // 向下掉头， 则要往上修正value
          value = this.Y - 10
        } else {
          value = this.Y + 10
        }
      }
      this.moveBody()
      this.head.style.top = value + 'px'
      // 检查头是否撞到了身体
      this.checkBreak()
    }

    // 增加身体
    addBody (){
      this.element.insertAdjacentHTML("beforeend", "<div></div>")
    }

    // 移动身体
    // 最后一节移到倒数第二节， 把倒数第二节移到倒数第三节 ... 从后开始移动
    // 蛇头不用管
    moveBody(){      
      for(let i = this.bodies.length - 1; i > 0; i--){
        // 前一截的位置
        let x = (this.bodies[i - 1] as HTMLElement).offsetLeft
        let y = (this.bodies[i - 1] as HTMLElement).offsetTop;
        
        // HTMLElement 对象表示 HTML 中的一个元素。
        // HTMLElement 对象继承了Element 对象的标准属性，也实现了一些非标准属性
        // HTMLElement 简单的看成Element的子类

        // 给当前的一截赋值
        (this.bodies[i] as HTMLElement).style.left = x + 'px';
        (this.bodies[i] as HTMLElement).style.top = y + 'px';
      }
    }

    // 检查撞身体
    checkBreak(){
      for(let i = 4; i < this.bodies.length; i++){
        let bd = this.bodies[i] as HTMLElement
        if(bd.offsetLeft === this.X && bd.offsetTop === this.Y){
          throw new Error('蛇撞到自己了')
        }
      }
    }
}

export default Snake